==> go-test-framework-212.5080.40-sources.jar: com/goide/vgo/VgoTestUtil.java <==
// Copyright 2016-2021 JetBrains s.r.o.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package com.goide.vgo;

import com.goide.GoConstants;
import com.goide.GoTestUtil;
import com.goide.project.GoModuleLibrariesService;
import com.goide.psi.*;
import com.goide.vendor.GoVendorExcludePolicy;
import com.goide.vgo.configuration.VgoProjectSettings;
import com.goide.vgo.mod.psi.VgoFile;
import com.goide.vgo.project.*;
import com.intellij.openapi.application.ReadAction;
import com.intellij.openapi.util.Disposer;
import com.intellij.openapi.util.Pair;
import com.intellij.openapi.util.io.FileUtil;
import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.vfs.VfsUtilCore;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.psi.PsiDirectory;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.PsiManager;
import com.intellij.testFramework.PlatformTestUtil;
import com.intellij.testFramework.TestModeFlags;
import com.intellij.testFramework.VfsTestUtil;
import com.intellij.testFramework.fixtures.CodeInsightTestFixture;
import com.intellij.util.ArrayUtil;
import com.intellij.util.PathUtil;
import com.intellij.util.containers.ContainerUtil;
import junit.framework.TestCase;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.nio.file.Paths;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class VgoTestUtil {
  private static final String GOPATH = GoTestUtil.getTestDataPath("vgo/mockGoPath").getAbsolutePath();
  private static final Map<String, VgoDependency> DEPENDENCIES =
    ContainerUtil.newHashMap(dependencyPair("vgoDep", "v1.5.2", null, null),
                             dependencyPair("vgoTransitiveDep", "v1.0.0", null, null),
                             dependencyPair("customDir", "v0.0.0", "customDirDep@v1.0.0", null),
                             dependencyPair("originalDep", "v1.0.0", null, dependency("replacedDep", "v1.0.0", null, null)));
  public static final String DEFAULT_IMPORT_PATH = "jetbrains.com/hello";

  @NotNull
  private static Pair<String, VgoDependencyImpl> dependencyPair(@NotNull String importPath,
                                                                @NotNull String version,
                                                                @Nullable String dir,
                                                                @Nullable VgoDependencyImpl replace) {
    VgoDependencyImpl dependency = dependency(importPath, version, dir, replace);
    return Pair.create(dependency.getDirPath(), dependency);
  }

  @NotNull
  private static VgoDependencyImpl dependency(@NotNull String importPath,
                                              @NotNull String version,
                                              @Nullable String dir,
                                              @Nullable VgoDependencyImpl replace) {
    String dirName = dir != null ? dir : String.format("%s@%s", importPath, version);
    String dirPath = FileUtil.join(GOPATH, "pkg", GoConstants.VGO_DIR_NAME, dirName);
    return new VgoDependencyImpl(importPath, version, null, PathUtil.toSystemIndependentName(FileUtil.toCanonicalPath(dirPath)), replace,
                                 null);
  }

  public static PsiFile setupVgoIntegration(@NotNull CodeInsightTestFixture fixture) {
    return setupVgoIntegration(null, DEFAULT_IMPORT_PATH, fixture, DEPENDENCIES);
  }

  public static PsiFile setupVgoIntegration(@Nullable String modulePath,
                                            @NotNull String importPath,
                                            @NotNull CodeInsightTestFixture fixture,
                                            @NotNull Map<String, VgoDependency> dependencies) {
    setupGopath(fixture, null);
    VgoModule module = createVgoModule(fixture, modulePath, importPath, dependencies);
    setupVgoIntegration(fixture, Collections.singletonList(module));
    return getGoModPsiFile(fixture, module);
  }

  @NotNull
  public static VirtualFile setupGopath(@NotNull CodeInsightTestFixture fixture, @Nullable String customGopath) {
    if (customGopath == null) {
      GoModuleLibrariesService.getInstance(fixture.getModule()).setLibraryRootUrls(VfsUtilCore.pathToUrl(GOPATH));
      return VfsTestUtil.findFileByCaseSensitivePath(GOPATH);
    }
    VirtualFile customGopathFile = VfsUtil.findFile(Paths.get(fixture.getTestDataPath(), customGopath), true);
    if (customGopathFile != null) {
      // Sometimes changes in custom gopath directory are not detected by tests, we explicitly refresh to overcome this.
      customGopathFile.refresh(false, true);
      GoModuleLibrariesService.getInstance(fixture.getModule()).setLibraryRootUrls(customGopathFile.getUrl());
      return customGopathFile;
    }
    throw new IllegalArgumentException("Cannot find custom gopath: " + customGopath);
  }

  @NotNull
  private static VgoModule createVgoModule(@NotNull CodeInsightTestFixture fixture,
                                           @Nullable String modulePath,
                                           @NotNull String importPath,
                                           @NotNull Map<String, VgoDependency> dependencies) {
    String goModPath = modulePath != null ? FileUtil.join(modulePath, VgoUtil.GO_MOD) : VgoUtil.GO_MOD;
    PsiFile file = fixture.addFileToProject(goModPath, "module \"" + importPath + "\"");
    return new VgoModule(file.getVirtualFile().getParent(), importPath, dependencies);
  }

  public static void setupVgoIntegration(@NotNull CodeInsightTestFixture fixture, @NotNull List<VgoModule> modules) {
    VgoExcludeRootsPolicy.setPointersDisposable(fixture.getTestRootDisposable());
    GoVendorExcludePolicy.setPointersDisposable(fixture.getTestRootDisposable());
    TestModeFlags.set(VgoIntegrationManager.DISABLE_TRACKERS, true, fixture.getTestRootDisposable());
    VgoProjectSettings.getInstance(fixture.getProject()).setIntegrationEnabled(true);
    setVgoModules(fixture, modules);
    Disposer.register(fixture.getTestRootDisposable(), () -> {
      VgoProjectSettings.getInstance(fixture.getProject()).setIntegrationEnabled(false);
      VgoModulesRegistry.getInstance(fixture.getProject()).updateAllModules(fixture.getModule(), Collections.emptySet());
      PlatformTestUtil.dispatchAllInvocationEventsInIdeEventQueue();
    });
    PlatformTestUtil.dispatchAllInvocationEventsInIdeEventQueue();
  }

  public static void setVgoModules(@NotNull CodeInsightTestFixture fixture, @NotNull List<VgoModule> modules) {
    for (VgoModule module : modules) {
      VgoModuleInfoProvider.VgoModuleInfoProviderForTests.putTestingVgoModule(module, fixture.getTestRootDisposable());
    }
    List<VgoRootToModule> vgoModules = ContainerUtil.map(modules, module -> new VgoRootToModule(module.getRoot(), module));
    VgoModulesRegistry.getInstance(fixture.getProject()).updateModules(fixture.getModule(), vgoModules);
  }

  @NotNull
  public static VgoFile getGoModPsiFile(@NotNull CodeInsightTestFixture fixture, @NotNull VgoModule vgoModule) {
    return (VgoFile)VgoTestUtil.getPsiFile(fixture, getGoModFile(vgoModule));
  }

  public static @NotNull VirtualFile getGoModFile(@NotNull VgoModule vgoModule) {
    return vgoModule.getRoot().findChild(VgoUtil.GO_MOD);
  }

  public static @Nullable PsiFile findPsiFileInTempDir(@NotNull CodeInsightTestFixture fixture, @NotNull String filePath) {
    VirtualFile file = fixture.findFileInTempDir(filePath);
    return file != null ? getPsiFile(fixture, file) : null;
  }

  public static @Nullable PsiFile getPsiFile(@NotNull CodeInsightTestFixture fixture, @NotNull VirtualFile virtualFile) {
    return ReadAction.compute(() -> PsiManager.getInstance(fixture.getProject()).findFile(virtualFile));
  }

  public static @NotNull String getTestDataPath(@NotNull String basePath) {
    return GoTestUtil.getTestDataPath("vgo/" + basePath).getAbsolutePath();
  }

  @NotNull
  public static GoReferenceExpressionBase findRefExpressionByText(@NotNull CodeInsightTestFixture fixture, @NotNull String refText) {
    return fixture.findElementByText(refText, GoReferenceExpressionBase.class);
  }

  @Nullable
  public static PsiElement resolveLastReference(@NotNull GoImportSpec importSpec) {
    return ArrayUtil.getLastElement(importSpec.getReferences()).resolve();
  }

  public static void assertResolved(@NotNull GoResolvable resolvable, @NotNull Class<?> clazz, @NotNull String pathSuffix) {
    assertResolved(resolvable.resolve(), clazz, pathSuffix);
  }

  public static void assertResolved(@Nullable PsiElement resolve, @NotNull Class<?> clazz, @NotNull String pathSuffix) {
    TestCase.assertNotNull("Not resolved", resolve);
    TestCase.assertTrue("Expected " + clazz.getSimpleName() + ", but got " + resolve.getClass().getSimpleName(), clazz.isInstance(resolve));
    VirtualFile virtualFile = resolve instanceof PsiDirectory ? ((PsiDirectory)resolve).getVirtualFile()
                                                              : resolve.getContainingFile().getVirtualFile();
    TestCase.assertTrue("Unexpected path", virtualFile.getPath().endsWith(pathSuffix));
  }

  @NotNull
  public static VirtualFile findModulesCacheFile(@NotNull CodeInsightTestFixture fixture, @NotNull String path) {
    return VgoUtil.getDependenciesRoot(fixture.getProject(), fixture.getModule()).findFileByRelativePath(path);
  }

  public static boolean resolveScopeContains(@NotNull GoFile scopeContext, @NotNull VirtualFile file) {
    GoNamedElement element = ContainerUtil.find(ContainerUtil.concat(scopeContext.getTypes(), scopeContext.getVars()), GoNamedElement::isPublic);
    TestCase.assertNotNull("Cannot find public named element", element);
    return element.getResolveScope().contains(file);
  }
}

==> go-test-framework-213.5744.269-sources.jar: com/goide/vgo/VgoTestUtil.java <==
// Copyright 2016-2021 JetBrains s.r.o.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package com.goide.vgo;

import com.goide.GoConstants;
import com.goide.GoTestUtil;
import com.goide.project.GoModuleLibrariesService;
import com.goide.psi.*;
import com.goide.vendor.GoVendorExcludePolicy;
import com.goide.vgo.configuration.VgoProjectSettings;
import com.goide.vgo.mod.psi.VgoFile;
import com.goide.vgo.project.*;
import com.intellij.openapi.application.ReadAction;
import com.intellij.openapi.util.Disposer;
import com.intellij.openapi.util.Pair;
import com.intellij.openapi.util.io.FileUtil;
import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.vfs.VfsUtilCore;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.psi.PsiDirectory;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.PsiManager;
import com.intellij.testFramework.PlatformTestUtil;
import com.intellij.testFramework.TestModeFlags;
import com.intellij.testFramework.VfsTestUtil;
import com.intellij.testFramework.fixtures.CodeInsightTestFixture;
import com.intellij.util.ArrayUtil;
import com.intellij.util.PathUtil;
import com.intellij.util.containers.ContainerUtil;
import junit.framework.TestCase;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.nio.file.Paths;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class VgoTestUtil {
  private static final String GOPATH = GoTestUtil.getTestDataPath("vgo/mockGoPath").getAbsolutePath();
  private static final Map<String, VgoDependency> DEPENDENCIES =
    ContainerUtil.newHashMap(dependencyPair("vgoDep", "v1.5.2", null, null),
                             dependencyPair("vgoTransitiveDep", "v1.0.0", null, null),
                             dependencyPair("customDir", "v0.0.0", "customDirDep@v1.0.0", null),
                             dependencyPair("originalDep", "v1.0.0", null, dependency("replacedDep", "v1.0.0", null, null)));
  public static final String DEFAULT_IMPORT_PATH = "jetbrains.com/hello";

  @NotNull
  private static Pair<String, VgoDependencyImpl> dependencyPair(@NotNull String importPath,
                                                                @NotNull String version,
                                                                @Nullable String dir,
                                                                @Nullable VgoDependencyImpl replace) {
    VgoDependencyImpl dependency = dependency(importPath, version, dir, replace);
    return Pair.create(dependency.getDirPath(), dependency);
  }

  @NotNull
  private static VgoDependencyImpl dependency(@NotNull String importPath,
                                              @NotNull String version,
                                              @Nullable String dir,
                                              @Nullable VgoDependencyImpl replace) {
    String dirName = dir != null ? dir : String.format("%s@%s", importPath, version);
    String dirPath = FileUtil.join(GOPATH, "pkg", GoConstants.VGO_DIR_NAME, dirName);
    return new VgoDependencyImpl(importPath, version, null, PathUtil.toSystemIndependentName(FileUtil.toCanonicalPath(dirPath)), replace,
                                 null);
  }

  public static PsiFile setupVgoIntegration(@NotNull CodeInsightTestFixture fixture) {
    return setupVgoIntegration(null, DEFAULT_IMPORT_PATH, fixture, DEPENDENCIES);
  }

  public static PsiFile setupVgoIntegration(@Nullable String modulePath,
                                            @NotNull String importPath,
                                            @NotNull CodeInsightTestFixture fixture,
                                            @NotNull Map<String, VgoDependency> dependencies) {
    setupGopath(fixture, null);
    VgoModule module = createVgoModule(fixture, modulePath, importPath, dependencies);
    setupVgoIntegration(fixture, Collections.singletonList(module));
    return getGoModPsiFile(fixture, module);
  }

  @NotNull
  public static VirtualFile setupGopath(@NotNull CodeInsightTestFixture fixture, @Nullable String customGopath) {
    if (customGopath == null) {
      GoModuleLibrariesService.getInstance(fixture.getModule()).setLibraryRootUrls(VfsUtilCore.pathToUrl(GOPATH));
      return VfsTestUtil.findFileByCaseSensitivePath(GOPATH);
    }
    VirtualFile customGopathFile = VfsUtil.findFile(Paths.get(fixture.getTestDataPath(), customGopath), true);
    if (customGopathFile != null) {
      // Sometimes changes in custom gopath directory are not detected by tests, we explicitly refresh to overcome this.
      customGopathFile.refresh(false, true);
      GoModuleLibrariesService.getInstance(fixture.getModule()).setLibraryRootUrls(customGopathFile.getUrl());
      return customGopathFile;
    }
    throw new IllegalArgumentException("Cannot find custom gopath: " + customGopath);
  }

  @NotNull
  private static VgoModule createVgoModule(@NotNull CodeInsightTestFixture fixture,
                                           @Nullable String modulePath,
                                           @NotNull String importPath,
                                           @NotNull Map<String, VgoDependency> dependencies) {
    String goModPath = modulePath != null ? FileUtil.join(modulePath, VgoUtil.GO_MOD) : VgoUtil.GO_MOD;
    PsiFile file = fixture.addFileToProject(goModPath, "module \"" + importPath + "\"");
    return new VgoModule(file.getVirtualFile().getParent(), importPath, dependencies);
  }

  public static void setupVgoIntegration(@NotNull CodeInsightTestFixture fixture, @NotNull List<VgoModule> modules) {
    VgoExcludeRootsPolicy.setPointersDisposable(fixture.getTestRootDisposable());
    GoVendorExcludePolicy.setPointersDisposable(fixture.getTestRootDisposable());
    TestModeFlags.set(VgoIntegrationManager.DISABLE_TRACKERS, true, fixture.getTestRootDisposable());
    VgoProjectSettings.getInstance(fixture.getProject()).setIntegrationEnabled(true);
    setVgoModules(fixture, modules);
    Disposer.register(fixture.getTestRootDisposable(), () -> {
      VgoProjectSettings.getInstance(fixture.getProject()).setIntegrationEnabled(false);
      VgoModulesRegistry.getInstance(fixture.getProject()).updateAllModules(fixture.getModule(), Collections.emptySet());
      PlatformTestUtil.dispatchAllInvocationEventsInIdeEventQueue();
    });
    PlatformTestUtil.dispatchAllInvocationEventsInIdeEventQueue();
  }

  public static void setVgoModules(@NotNull CodeInsightTestFixture fixture, @NotNull List<VgoModule> modules) {
    for (VgoModule module : modules) {
      VgoModuleInfoProviderForTests.putTestingVgoModule(module, fixture.getTestRootDisposable());
    }
    List<VgoRootToModule> vgoModules = ContainerUtil.map(modules, module -> new VgoRootToModule(module.getRoot(), module));
    VgoModulesRegistry.getInstance(fixture.getProject()).updateModules(fixture.getModule(), vgoModules);
  }

  @NotNull
  public static VgoFile getGoModPsiFile(@NotNull CodeInsightTestFixture fixture, @NotNull VgoModule vgoModule) {
    return (VgoFile)VgoTestUtil.getPsiFile(fixture, getGoModFile(vgoModule));
  }

  public static @NotNull VirtualFile getGoModFile(@NotNull VgoModule vgoModule) {
    return vgoModule.getRoot().findChild(VgoUtil.GO_MOD);
  }

  public static @Nullable PsiFile findPsiFileInTempDir(@NotNull CodeInsightTestFixture fixture, @NotNull String filePath) {
    VirtualFile file = fixture.findFileInTempDir(filePath);
    return file != null ? getPsiFile(fixture, file) : null;
  }

  public static @Nullable PsiFile getPsiFile(@NotNull CodeInsightTestFixture fixture, @NotNull VirtualFile virtualFile) {
    return ReadAction.compute(() -> PsiManager.getInstance(fixture.getProject()).findFile(virtualFile));
  }

  public static @NotNull String getTestDataPath(@NotNull String basePath) {
    return GoTestUtil.getTestDataPath("vgo/" + basePath).getAbsolutePath();
  }

  @NotNull
  public static GoReferenceExpressionBase findRefExpressionByText(@NotNull CodeInsightTestFixture fixture, @NotNull String refText) {
    return fixture.findElementByText(refText, GoReferenceExpressionBase.class);
  }

  @Nullable
  public static PsiElement resolveLastReference(@NotNull GoImportSpec importSpec) {
    return ArrayUtil.getLastElement(importSpec.getReferences()).resolve();
  }

  public static void assertResolved(@NotNull GoResolvable resolvable, @NotNull Class<?> clazz, @NotNull String pathSuffix) {
    assertResolved(resolvable.resolve(), clazz, pathSuffix);
  }

  public static void assertResolved(@Nullable PsiElement resolve, @NotNull Class<?> clazz, @NotNull String pathSuffix) {
    TestCase.assertNotNull("Not resolved", resolve);
    TestCase.assertTrue("Expected " + clazz.getSimpleName() + ", but got " + resolve.getClass().getSimpleName(), clazz.isInstance(resolve));
    VirtualFile virtualFile = resolve instanceof PsiDirectory ? ((PsiDirectory)resolve).getVirtualFile()
                                                              : resolve.getContainingFile().getVirtualFile();
    TestCase.assertTrue("Unexpected path", virtualFile.getPath().endsWith(pathSuffix));
  }

  @NotNull
  public static VirtualFile findModulesCacheFile(@NotNull CodeInsightTestFixture fixture, @NotNull String path) {
    return VgoUtil.getDependenciesRoot(fixture.getProject(), fixture.getModule()).findFileByRelativePath(path);
  }

  public static boolean resolveScopeContains(@NotNull GoFile scopeContext, @NotNull VirtualFile file) {
    GoNamedElement element = ContainerUtil.find(ContainerUtil.concat(scopeContext.getTypes(), scopeContext.getVars()), GoNamedElement::isPublic);
    TestCase.assertNotNull("Cannot find public named element", element);
    return element.getResolveScope().contains(file);
  }
}
